
#include<stdio.h>
#include<assert.h>
#include<string.h>

#include"nb_key.h"
#include"calibration.h"

#include<HD/hd.h>
#include<HDU/hduError.h>
#include<HDU/hduVector.h>

/*
 * Program tworzy pozioma plaszczyzne, ktora oddzialuje sila tarcia.
 */

HDCallbackCode HDCALLBACK surfaceCallback(void *data);

int main(int argc, char* argv[])
{    
    HDErrorInfo error;
    HDSchedulerHandle hSurface;

    /* Inicjalizacja urzadzenia. Od tego zaczynamy */

    HHD hHD = hdInitDevice(HD_DEFAULT_DEVICE);
    if (HD_DEVICE_ERROR(error = hdGetError())) {
        hduPrintError(stderr, &error, "Failed to initialize haptic device");
        return -1;
    }

	/* Udalo sie. Urzadzenie jest i dziala. */
    printf("Hello Haptic Device!\n");
    printf("Found device model: %s.\n\n", hdGetString(HD_DEVICE_MODEL_TYPE));

	
	/* Start petli urzadzenia */
    hdEnable(HD_FORCE_OUTPUT);
    hdStartScheduler();

    /* Czy sie udalo? */
    if (HD_DEVICE_ERROR(error = hdGetError()))
    {
        hduPrintError(stderr, &error, "Failed to start scheduler");
        fprintf(stderr, "\nPress any key to quit.\n");
        return -1;
    }

	/* Kalibracja urzadzenia. Co bedzie, jesli sie ja wylaczy? */
	calibrate_haptic();

    /* Instalujemy callback. 0 oznacza pusty wskaznik do danych, ktore moglyby byc zwrocone. */
    hSurface = hdScheduleAsynchronous(surfaceCallback, 0, HD_MAX_SCHEDULER_PRIORITY);
    printf("Feel around for the surface...\n");
    printf("Press any key to quit.\n\n");

    int done=0;
 
    /* Przestawiamy terminal w tryb nieblokujacy.  */
    nonblock(NB_ENABLE);
    while(!done) {
        usleep(100000);
        int i=kbhit();
		if(i!=0) { /* Uzytkownik nacisnal klawisz. Koniec. */
			break;
		}
        /* Sprawdzamy, czy callback sie nie zakonczyl */
        if (!hdWaitForCompletion(hSurface, HD_WAIT_CHECK_STATUS)) {
            fprintf(stderr, "Callback terminated.\n");     
            break;
        }
    }
    nonblock(NB_DISABLE);


    /* Zatrzymanie schedulera i usuniecie callback'u */
    hdStopScheduler();
    hdUnschedule(hSurface);

    /* Wylaczenie urzadzenia. */
    hdDisableDevice(hHD);

    return 0;
}

/*
 * Callback, ktory jest wywolywany w kazdej iteracji petli schedulera.
 */
HDCallbackCode HDCALLBACK surfaceCallback(void *data)
{
    const HDdouble maxForce=3; /* N */
	const HDdouble stiffness=0.5; /* N/mm */

    static int direction=1; /* Kierunek, w ktorym plaszczyzna jest przenikalna */

    HDErrorInfo error;
    hduVector3Dd position;

    HHD hHD = hdGetCurrentDevice();

	/* Poczatek ramki haptycznej */
    hdBeginFrame(hHD);

    /* Biezaca pozycja urzadzenia */
    hdGetDoublev(HD_CURRENT_POSITION, position);
    
	HDdouble f=0;

	/*
	 * position[1] to pozycja urzadzenia wzgledem osi Y
	 *
	 * Symulujemy plaszczyzne pozioma na wysokosci 0.
	 */
	if(position[1]*direction<0) { /* Iloczyn ujemny oznacza probe przebicia plaszczyzny. */
		f=fabs(position[1])*stiffness; /* Sila sprezystosci */

		if(f>maxForce) { /* Jezeli sila wieksza od maxForce, plaszczyzna zostaje przebita. */
			f=0;
			direction*=-1; /* Odwracamy kierunek, bo kursor znalazl sie po drugiej stronie. */
		}
	}

    hduVector3Dd force={0, 0, 0};

	if(f>0 ) { /* Jest nacisk. Liczymy tarcie. */
		hduVector3Dd velocity;

		/* Obliczamy predkosc. */
		hduVecSubtract(velocity, position, last_position);

		velocity[1]=0; // Interesuje nas predkosc wzgledem plaszczyzny.

		double modV=hduVecMagnitude(velocity);

		if(modV>0) {
			hduVecScale(force, velocity, -f*friction/modV);
		}
	}

	/* Dodajemy odpychanie od plaszczyzny. */
	force[1]=f*direction; 

    /* Aktualizacja sily. */
    hdSetDoublev(HD_CURRENT_FORCE, force);
    
    /* Koniec ramki. */
    hdEndFrame(hHD);

    /* Sprawdzenie, czy nie wystapil blad. */
    if (HD_DEVICE_ERROR(error = hdGetError())) {
        hduPrintError(stderr, &error, "Error detected while rendering gravity well\n");
        
        if (hduIsSchedulerError(&error)) {
            return HD_CALLBACK_DONE;
        }
    }

    /* Ten callback ma byc wywolany rowniez w nastepnej iteracji schedulera. */
    return HD_CALLBACK_CONTINUE;
}
