#include <stdio.h>
#include <assert.h>

# include <unistd.h>

#include <HD/hd.h>
#include <HDU/hduError.h>
#include <HDU/hduVector.h>

HDCallbackCode HDCALLBACK UpdateCalibrationCallback(void *pUserData);
HDCallbackCode HDCALLBACK CalibrationStatusCallback(void *pUserData);

HDenum GetCalibrationStatus();
HDboolean CheckCalibration(HDenum calibrationStyle);

void calibrate_haptic(void)
{
    int calibrationStyle = HD_CALIBRATION_INKWELL;

    if (calibrationStyle  == HD_CALIBRATION_INKWELL) {
        if (GetCalibrationStatus() == HD_CALIBRATION_NEEDS_MANUAL_INPUT) {
            printf("Please place the device into the inkwell ");
            printf("for calibration.\n\n");
        }
    }

    while(1) {
        if (CheckCalibration(calibrationStyle)) {
        	printf("Device calibrated.\n");
			break;
        }
        sleep(1);
    }
}

HDCallbackCode HDCALLBACK UpdateCalibrationCallback(void *pUserData)
{
    HDenum *calibrationStyle = (int *) pUserData;

	
    hdBeginFrame(hdGetCurrentDevice());
    if (hdCheckCalibration() == HD_CALIBRATION_NEEDS_UPDATE) {
        hdUpdateCalibration(*calibrationStyle);
    }

    hdEndFrame(hdGetCurrentDevice());
    return HD_CALLBACK_DONE;
}

HDCallbackCode HDCALLBACK CalibrationStatusCallback(void *pUserData)
{
    HDenum *pStatus = (HDenum *) pUserData;

    hdBeginFrame(hdGetCurrentDevice());
    *pStatus = hdCheckCalibration();
    hdEndFrame(hdGetCurrentDevice());

    return HD_CALLBACK_DONE;
}


HDenum GetCalibrationStatus()
{
    HDenum status;
    hdScheduleSynchronous(CalibrationStatusCallback, &status, HD_DEFAULT_SCHEDULER_PRIORITY);
    return status;
}

HDboolean CheckCalibration(HDenum calibrationStyle)
{
    HDenum status = GetCalibrationStatus();
    
    if (status == HD_CALIBRATION_OK) {
        return HD_TRUE;
    } else if (status == HD_CALIBRATION_NEEDS_MANUAL_INPUT) {
        printf("Calibration requires manual input...\n");
        return HD_FALSE;
    } else if (status == HD_CALIBRATION_NEEDS_UPDATE) {
        hdScheduleSynchronous(UpdateCalibrationCallback, &calibrationStyle,
            HD_DEFAULT_SCHEDULER_PRIORITY);

        if (HD_DEVICE_ERROR(hdGetError())) {
            printf("\nFailed to update calibration.\n\n");
            return HD_FALSE;
        } else {
            printf("\nCalibration updated successfully.\n\n");
            return HD_TRUE;
        }
    } else {
        assert(!"Unknown calibration status");
        return HD_FALSE;
    }
}

